@@ -1,24 +1,33 @@
 import gc
 import logging
-import sys
-from typing import Type, Tuple, Optional
-
 import os
-from abc import ABCMeta, abstractmethod, abstractproperty
+import sys
+from abc import ABCMeta
+from abc import abstractmethod
+from abc import abstractproperty
 from collections import defaultdict
 from importlib.util import find_spec
-from jsoncfg.value_mappers import require_string, require_array
+from typing import Optional
+from typing import Tuple
+from typing import Type
+
+from jsoncfg.value_mappers import require_array
+from jsoncfg.value_mappers import require_string
+from peek_platform import PeekPlatformConfig
 from twisted.internet.defer import inlineCallbacks
 
-from peek_platform import PeekPlatformConfig
 from peek_plugin_base.PluginCommonEntryHookABC import PluginCommonEntryHookABC
 from peek_plugin_base.PluginPackageFileConfig import PluginPackageFileConfig
 from vortex.PayloadIO import PayloadIO
-from vortex.Tuple import removeTuplesForTupleNames, registeredTupleNames, \
-    tupleForTupleName
-from vortex.TupleAction import TupleGenericAction, TupleUpdateAction
+from vortex.Tuple import registeredTupleNames
+from vortex.Tuple import removeTuplesForTupleNames
+from vortex.Tuple import tupleForTupleName
+from vortex.TupleAction import TupleGenericAction
+from vortex.TupleAction import TupleUpdateAction
 from vortex.TupleSelector import TupleSelector
-from vortex.rpc.RPC import _VortexRPCResultTuple, _VortexRPCArgTuple
+from vortex.VortexUtil import _DebounceArgsTuple
+from vortex.rpc.RPC import _VortexRPCArgTuple
+from vortex.rpc.RPC import _VortexRPCResultTuple
 
 logger = logging.getLogger(__name__)
 
@@ -28,12 +37,15 @@ TupleUpdateAction()
 TupleGenericAction()
 _VortexRPCResultTuple()
 _VortexRPCArgTuple()
+_DebounceArgsTuple()
 
 corePlugins = [
     "peek_core_email",
     "peek_core_device",
     "peek_core_search",
-    "peek_core_user"
+    "peek_core_user",
+    "peek_core_docdb",
+    "peek_core_screen",
 ]
 
 
@@ -41,7 +53,9 @@ class PluginLoaderABC(metaclass=ABCMeta):
     _instance = None
 
     def __new__(cls, *args, **kwargs):
-        assert cls._instance is None, "PluginServerLoader is a singleton, don't construct it"
+        assert (
+            cls._instance is None
+        ), "PluginServerLoader is a singleton, don't construct it"
         cls._instance = object.__new__(cls)
         return cls._instance
 
@@ -53,30 +67,30 @@ class PluginLoaderABC(metaclass=ABCMeta):
 
     @abstractproperty
     def _entryHookFuncName(self) -> str:
-        """ Entry Hook Func Name.
+        """Entry Hook Func Name.
         Protected property
-        :return: EG  "peekServerEntryHook"
+        :return: EG  "peekLogicEntryHook"
 
         """
 
     @abstractproperty
     def _entryHookClassType(self):
-        """ Entry Hook Class Type
+        """Entry Hook Class Type
         Protected property
-        :return: EG  PluginServerEntryHookABC
+        :return: EG  PluginLogicEntryHookABC
 
         """
 
     @abstractproperty
     def _platformServiceNames(self) -> [str]:
-        """ Platform Service Name
+        """Platform Service Name
         Protected property
-        :return: one or more of "server", "worker", "agent", "client", "storage"
+        :return: one or more of "logic", "worker", "agent", "client", "storage"
 
         """
 
     def pluginEntryHook(self, pluginName) -> Optional[PluginCommonEntryHookABC]:
-        """ Plugin Entry Hook
+        """Plugin Entry Hook
 
         Returns the loaded plugin entry hook for the plugin name.
 
@@ -93,7 +107,9 @@ class PluginLoaderABC(metaclass=ABCMeta):
         # make sure we don't load them twice, because we don't yet
         # support it.
         if pluginName in self._loadedPlugins:
-            raise Exception("Plugin %s is already loaded, check config.json" % pluginName)
+            raise Exception(
+                "Plugin %s is already loaded, check config.json" % pluginName
+            )
 
         try:
             self.unloadPlugin(pluginName)
@@ -104,42 +120,57 @@ class PluginLoaderABC(metaclass=ABCMeta):
 
             modSpec = find_spec(pluginName)
             if not modSpec:
-                raise Exception("Failed to find package %s,"
-                                " is the python package installed?" % pluginName)
+                raise Exception(
+                    "Failed to find package %s,"
+                    " is the python package installed?" % pluginName
+                )
 
             PluginPackage = modSpec.loader.load_module()
             pluginRootDir = os.path.dirname(PluginPackage.__file__)
 
             # Load up the plugin package info
             pluginPackageJson = PluginPackageFileConfig(pluginRootDir)
-            pluginVersion = pluginPackageJson.config.plugin.version(require_string)
+            pluginVersion = pluginPackageJson.config.plugin.version(
+                require_string)
             pluginRequiresService = pluginPackageJson.config.requiresServices(
-                require_array)
+                require_array
+            )
 
             # Make sure the service is required
             # Storage and Server are loaded at the same time, hence the intersection
             if not set(pluginRequiresService) & set(self._platformServiceNames):
-                logger.debug("%s does not require %s, Skipping load",
-                             pluginName, self._platformServiceNames)
+                logger.debug(
+                    "%s does not require %s, Skipping load",
+                    pluginName,
+                    self._platformServiceNames,
+                )
                 return
 
             # Get the entry hook class from the package
-            entryHookGetter = getattr(PluginPackage, str(self._entryHookFuncName))
+            entryHookGetter = getattr(PluginPackage,
+                str(self._entryHookFuncName))
             EntryHookClass = entryHookGetter() if entryHookGetter else None
 
             if not EntryHookClass:
                 logger.warning(
                     "Skipping load for %s, %s.%s is missing or returned None",
-                    pluginName, pluginName, self._entryHookFuncName)
+                    pluginName,
+                    pluginName,
+                    self._entryHookFuncName,
+                )
                 return
 
             if not issubclass(EntryHookClass, self._entryHookClassType):
-                raise Exception("%s load error, Excpected %s, received %s"
-                                % (pluginName, self._entryHookClassType, EntryHookClass))
+                raise Exception(
+                    "%s load error, Excpected %s, received %s"
+                    % (pluginName, self._entryHookClassType, EntryHookClass)
+                )
 
             ### Perform the loading of the plugin
-            yield self._loadPluginThrows(pluginName, EntryHookClass,
-                                         pluginRootDir, tuple(pluginRequiresService))
+            yield self._loadPluginThrows(
+                pluginName, EntryHookClass, pluginRootDir,
+                tuple(pluginRequiresService)
+            )
 
             # Make sure the version we have recorded is correct
             # JJC Disabled, this is just spamming the config file at the moment
@@ -147,10 +178,12 @@ class PluginLoaderABC(metaclass=ABCMeta):
 
             # Make note of the final registrations for this plugin
             self._vortexEndpointInstancesByPluginName[pluginName] = list(
-                set(PayloadIO().endpoints) - endpointInstancesBefore)
+                set(PayloadIO().endpoints) - endpointInstancesBefore
+            )
 
             self._vortexTupleNamesByPluginName[pluginName] = list(
-                set(registeredTupleNames()) - tupleNamesBefore)
+                set(registeredTupleNames()) - tupleNamesBefore
+            )
 
             self.sanityCheckServerPlugin(pluginName)
 
@@ -159,11 +192,14 @@ class PluginLoaderABC(metaclass=ABCMeta):
             logger.exception(e)
 
     @abstractmethod
-    def _loadPluginThrows(self, pluginName: str,
-                          EntryHookClass: Type[PluginCommonEntryHookABC],
-                          pluginRootDir: str,
-                          requiresService: Tuple[str, ...]) -> None:
-        """ Load Plugin (May throw Exception)
+    def _loadPluginThrows(
+        self,
+        pluginName: str,
+        EntryHookClass: Type[PluginCommonEntryHookABC],
+        pluginRootDir: str,
+        requiresService: Tuple[str, ...],
+    ) -> None:
+        """Load Plugin (May throw Exception)
 
         This method is called to perform the load of the module.
 
@@ -188,7 +224,8 @@ class PluginLoaderABC(metaclass=ABCMeta):
         del self._vortexEndpointInstancesByPluginName[pluginName]
 
         # Remove the registered tuples
-        removeTuplesForTupleNames(self._vortexTupleNamesByPluginName[pluginName])
+        removeTuplesForTupleNames(
+            self._vortexTupleNamesByPluginName[pluginName])
         del self._vortexTupleNamesByPluginName[pluginName]
 
         self._unloadPluginPackage(pluginName)
@@ -267,8 +304,9 @@ class PluginLoaderABC(metaclass=ABCMeta):
             if pluginName in self._loadedPlugins:
                 self.unloadPlugin(pluginName)
 
-        remainingOptionalPlugins = list(filter(lambda n: not n.startswith("peek_core"),
-                                               self._loadedPlugins))
+        remainingOptionalPlugins = list(
+            filter(lambda n: not n.startswith("peek_core"), self._loadedPlugins)
+        )
 
         if remainingOptionalPlugins:
             logger.debug(remainingOptionalPlugins)
@@ -283,8 +321,10 @@ class PluginLoaderABC(metaclass=ABCMeta):
             return plugin.start()
 
         except Exception as e:
-            logger.error("An exception occured while starting plugin %s,"
-                         " starting continues" % pluginName)
+            logger.error(
+                "An exception occured while starting plugin %s,"
+                " starting continues" % pluginName
+            )
             logger.exception(e)
 
     def _tryStop(self, pluginName):
@@ -293,8 +333,10 @@ class PluginLoaderABC(metaclass=ABCMeta):
             return plugin.stop()
 
         except Exception as e:
-            logger.error("An exception occured while stopping plugin %s,"
-                         " stopping continues" % pluginName)
+            logger.error(
+                "An exception occured while stopping plugin %s,"
+                " stopping continues" % pluginName
+            )
             logger.exception(e)
 
     def _unloadPluginPackage(self, pluginName):
@@ -308,14 +350,18 @@ class PluginLoaderABC(metaclass=ABCMeta):
             oldLoadedPlugin.unload()
 
         except Exception as e:
-            logger.error("An exception occured while unloading plugin %s,"
-                         " unloading continues" % pluginName)
+            logger.error(
+                "An exception occured while unloading plugin %s,"
+                " unloading continues" % pluginName
+            )
             logger.exception(e)
 
         # Unload the packages
-        loadedSubmodules = [modName
-                            for modName in list(sys.modules.keys())
-                            if modName.startswith('%s.' % pluginName)]
+        loadedSubmodules = [
+            modName
+            for modName in list(sys.modules.keys())
+            if modName.startswith("%s." % pluginName)
+        ]
 
         for modName in loadedSubmodules:
             del sys.modules[modName]
@@ -328,34 +374,44 @@ class PluginLoaderABC(metaclass=ABCMeta):
         # pypy doesn't have getrefcount
         # ("oldLoadedPlugin" in this method and the call to getrefcount) == 2 references
         if hasattr(sys, "getrefcount") and sys.getrefcount(oldLoadedPlugin) > 2:
-            logger.warning("Old references to %s still exist, count = %s",
-                           pluginName, sys.getrefcount(oldLoadedPlugin))
+            logger.warning(
+                "Old references to %s still exist, count = %s",
+                pluginName,
+                sys.getrefcount(oldLoadedPlugin),
+            )
 
         del oldLoadedPlugin
         gc.collect()
         # Now there should be no references.
 
     def sanityCheckServerPlugin(self, pluginName):
-        ''' Sanity Check Plugin
+        """Sanity Check Plugin
 
         This method ensures that all the things registed for this plugin are
         prefixed by it's pluginName, EG plugin_noop
-        '''
+        """
 
         # All endpoint filters must have the 'plugin' : 'plugin_name' in them
         for endpoint in self._vortexEndpointInstancesByPluginName[pluginName]:
             filt = endpoint.filt
-            if 'plugin' not in filt and filt['plugin'] != pluginName:
-                raise Exception("Payload endpoint does not contan 'plugin':'%s'\n%s"
-                                % (pluginName, filt))
+            if "plugin" not in filt and filt["plugin"] != pluginName:
+                raise Exception(
+                    "Payload endpoint does not contain 'plugin':'%s'\n%s"
+                    % (pluginName, filt)
+                )
 
         # all tuple names must start with their pluginName
         for tupleName in self._vortexTupleNamesByPluginName[pluginName]:
             TupleCls = tupleForTupleName(tupleName)
             if not tupleName.startswith(pluginName):
-                raise Exception("Tuple name does not start with '%s', %s (%s)"
-                                % (pluginName, tupleName, TupleCls.__name__))
+                raise Exception(
+                    "Tuple name does not start with '%s', %s (%s)"
+                    % (pluginName, tupleName, TupleCls.__name__)
+                )
 
     def notifyOfPluginVersionUpdate(self, pluginName, pluginVersion):
-        logger.info("Received PLUGIN update for %s version %s", pluginName, pluginVersion)
+        logger.info(
+            "Received PLUGIN update for %s version %s", pluginName,
+            pluginVersion
+        )
         return self.loadPlugin(pluginName)
